@@ -33,6 +33,12 @@ class Arkivist:
         self.sort = sort
         self.reverse = reverse
         self.autosave = autosave
+        ## query
+        self.operator = "="
+        self.child_key = None
+        self.child_val = None
+        self.case_sensitive = True
+        self.matches = {}
     
     def update(self, data):
         """
@@ -43,14 +49,15 @@ class Arkivist:
             data: dictionary
                 dictionary object
         """
+        self.child_key = None
         self.collection.update(data)
         if self.autosave and self.save_to_file:
             update_file(self.filepath, self.collection, self.indent, self.sort, self.reverse)
         return self
     
     def set(self, data):
-        """ Duplicate for update function. """
-        return self.update(data)
+        print("Deprecated function, use update() instead.")
+        return self
 
     def fetch(self, url):
         """
@@ -61,6 +68,7 @@ class Arkivist:
             url: str
                 any valid URL string
         """
+        self.child_key = None
         try:
             with requests.get(url) as source:
                 self.collection = source.json()
@@ -78,6 +86,7 @@ class Arkivist:
             data: str / dict
                 any valid string or dict object
         """
+        self.child_key = None
         if type(data) == dict:
             self.replace(data)
         elif type(data) == str:
@@ -90,6 +99,7 @@ class Arkivist:
     
     def invert(self):
         """ Inverts the dictionary """
+        self.child_key = None
         try:
             # hashable keys / values
             self.collection = dict(zip(self.collection.values(), self.collection.keys()))
@@ -101,6 +111,7 @@ class Arkivist:
     
     def flatten(self):
         """  Flattens a nested dictionary. """
+        self.child_key = None
         self.collection = flattener(self.collection)
         return self
     
@@ -117,11 +128,33 @@ class Arkivist:
         """
         self.sort = sort
         self.reverse = reverse
+        self.matches = query(self.collection, self.child_key, self.child_val, self.operator, self.case_sensitive)
+        self.child_key = None
         if sort:
-            self.collection = dict(sorted(self.collection.items(), reverse=reverse))
-        return self.collection
+            self.matches = dict(sorted(self.matches.items(), reverse=reverse))
+        return self.matches
     
-    def search(self, key, fallback=None):
+    def items(self, sort=False, reverse=False):
+        """
+            Generator for dictionary
+            ...
+            Parameters
+            ---
+            sort: boolean
+                sorts the dictionary based on keys
+            reverse: boolean
+                sorts the dictionary in reverse
+        """
+        self.sort = sort
+        self.reverse = reverse
+        self.matches = query(self.collection, self.child_key, self.child_val, self.operator, self.case_sensitive)
+        self.child_key = None
+        if sort:
+            self.matches = dict(sorted(self.matches.items(), reverse=reverse))
+        for key, value in self.matches.items():
+            yield key, value
+    
+    def get(self, key, fallback=None):
         """
             Gets the value of the search key from the  dictionary.
             ...
@@ -133,34 +166,106 @@ class Arkivist:
                 any datatype to return when the key doesn't exist
                 
         """
+        self.child_key = None
         return self.collection.get(key, fallback)
     
+    def search(self, key, fallback=None):
+        """ Duplicate of get function """
+        self.child_key = None
+        print("Deprected function, use get() instead.")
+        return self
+    
+    def where(self, child, value=None, case_sensitive=True):
+        """
+            Gets the value of the search key from the  dictionary.
+            ...
+            Parameters
+            ---
+            child: string
+                child key found in the dictionary
+            value: string, int, float
+                value of the child key
+            case_sensitive: boolean
+                if search operation is case sensitive
+                
+        """
+        self.operator = "="
+        self.child_key = child
+        self.child_val = value
+        self.case_sensitive = case_sensitive
+        self.matches = {}
+        return self
+    
+    def contains(self, value, case_sensitive=True):
+        """
+            Gets the value of the search key from the  dictionary.
+            ...
+            Parameters
+            ---
+            value: string, int, float
+                value of the child key
+            case_sensitive: boolean
+                if search operation is case sensitive
+                
+        """
+        if self.child_key != None:
+            self.operator = "%"
+            self.child_val = value
+            self.case_sensitive = case_sensitive
+            self.matches = {}
+        return self
+    
+    def exclude(self, value, case_sensitive=True):
+        """
+            Gets the value of the search key from the  dictionary.
+            ...
+            Parameters
+            ---
+            value: string, int, float
+                value of the child key
+            case_sensitive: boolean
+                if search operation is case sensitive
+                
+        """
+        if self.child_key != None:
+            self.operator = "x"
+            self.child_val = value
+            self.case_sensitive = case_sensitive
+            self.matches = {}
+        return self
+    
     def keys(self):
         """ Get all the keys of the ditionary """
+        self.child_key = None
         return list(self.collection.keys())
     
     def values(self):
         """ Get all the values of the ditionary """
+        self.child_key = None
         return list(self.collection.values())
     
     def is_empty(self):
         """ Check if dictionary is empty """
+        self.child_key = None
         if len(self.collection) == 0:
             return True
         return False
     
     def is_not_empty(self):
         """ Check if dictionary is not empty """
+        self.child_key = None
         if len(self.collection) == 0:
             return False
         return True
     
     def count(self):
         """ Count the number of entries in the dictionary """
+        self.child_key = None
         return len(self.collection)
     
     def clear(self):
         """ Clears the dictionary """
+        self.child_key = None
         self.collection = {}
         if self.autosave and self.save_to_file:
             update_file(self.filepath, self.collection, self.indent, self.sort, self.reverse)
@@ -175,6 +280,7 @@ class Arkivist:
             collection: dict
                 a dictionary to replace the contents of the original dictionary
         """
+        self.child_key = None
         if type(collection) == dict:
             self.collection = collection
             if self.autosave and self.save_to_file:
@@ -192,6 +298,7 @@ class Arkivist:
             filepath: string
                 any filepath, if empty defaults to previously set filepath
         """
+        self.child_key = None
         self.save_to_file = True
         savepath = self.filepath
         if filepath != "": savepath = filepath
@@ -229,6 +336,38 @@ def flattener(data):
     flatten(data)
     return out
 
+def query(collection, child_key, child_val, operator, case_sensitive):
+    matches = {}
+    if child_key == None:
+        matches = collection
+    else:
+        for key, data in collection.items():
+            value = data.get(child_key, "")
+            if operator == "=":
+                if case_sensitive:
+                    if child_val == value:
+                        matches.update({key: data})
+                else:
+                    if str(child_val).lower() == str(value).lower():
+                        matches.update({key: data})
+            elif operator == "%":
+                if case_sensitive:
+                    if child_val in value:
+                        matches.update({key: data})
+                else:
+                    # if case sensitive is false, convert values to string
+                    if str(child_val).lower() in str(value).lower():
+                        matches.update({key: data})
+            elif operator == "x":
+                if case_sensitive:
+                    if child_val not in value:
+                        matches.update({key: data})
+                else:
+                    # if case sensitive is false, convert values to string
+                    if str(child_val).lower() not in str(value).lower():
+                        matches.update({key: data})
+    return matches
+
 def update_file(filepath, data, indent=4, sort=False, reverse=False):
     """
         Formats and saves the dictionary into a JSON file.
